# -*- coding: utf-8 -*-
from ctypes import *
import os
import platform


class API:
    api = None

    def __init__(self, ip="", port=-1):
        pf = platform.system()

        # Load API module
        try:

            # 64bit API
            if pf == 'Windows':
                self.api = windll.LoadLibrary(os.getcwd() + "\\mxnet2api.dll")
            elif pf == "Darwin":
                self.api = cdll.LoadLibrary("/usr/local/lib/libmxnet2api.dylib")
            else:
                self.api = cdll.LoadLibrary("/usr/lib/libmxnet2api.so")

        except Exception as e:
            print(e)
            self.api = None
            return

        # define argument types for MxNet2 API functions from mxnet2api.dll
        self.api.rInit_MatrixAPI.restype = c_short
        self.api.rDongle_Count.restype = c_short
        self.api.rDongle_Count.argtypes = [c_short]
        self.api.rDongle_MemSize.argtypes = [c_short, c_short]
        self.api.rDongle_Model.argtypes = [c_short, c_short]
        self.api.rDongle_Version.argtypes = [c_short, c_short]
        self.api.rDongle_ReadData.restype = c_short
        self.api.rDongle_ReadData.argtypes = [c_int32, POINTER(c_int32), c_short, c_short, c_short]
        self.api.rDongle_ReadDataEx.restype = c_short
        self.api.rDongle_ReadDataEx.argtypes = [c_int32, POINTER(c_int32), c_short, c_short, c_short, c_short]
        self.api.rDongle_ReadSerNr.restype = c_int32
        self.api.rDongle_ReadSerNr.argtypes = [c_int32, c_short, c_short]
        self.api.rDongle_GetKeyFlag.argtypes = [c_int32, c_short, c_short]
        self.api.rDongle_EncryptData.restype = c_short
        self.api.rDongle_EncryptData.argtypes = [c_int32, POINTER(c_int32), c_short, c_short]
        self.api.rDongle_DecryptData.restype = c_short
        self.api.rDongle_DecryptData.argtypes = [c_int32, POINTER(c_int32), c_short, c_short]
        self.api.rDongle_GetDriverFlag.argtypes = [c_int32, c_short, c_short]
        self.api.rSetConfig_MatrixNet.argtypes = [c_short, c_char_p]
        self.api.rGetConfig_MatrixNet.argtypes = [c_short]
        self.api.rLogIn_MatrixNet.restype = c_short
        self.api.rLogIn_MatrixNet.argtypes = [c_int32, c_short, c_short]
        self.api.rLogOut_MatrixNet.restype = c_short
        self.api.rLogOut_MatrixNet.argtypes = [c_int32, c_short, c_short]

        if pf == 'Windows':
            self.api.MxNet_SetIP.restype = c_wchar_p
            self.api.MxNet_SetIP.argtypes = [c_wchar_p]
        else:
            self.api.MxNet_SetIP.restype = c_char_p
            self.api.MxNet_SetIP.argtypes = [c_char_p]

        self.api.MxNet_SetPort.restype = c_int32
        self.api.MxNet_SetPort.argtypes = [c_int32]

        if ip == "no-connect":
            return

        if ip != "":
            if pf=='Windows':
            	setip = wstring_at(self.set_ip(ip))
            else:
                setip = str(self.set_ip(ip),"utf-8")

            if setip == "":
                print("*** failed to set IP. IP willl be read from mxnetapi.ini ***")
            else:
                if port > 0:
                    self.set_port(port)

        self.connect()

    def __del__(self):
        if self.api is None:
            return -1

        self.api.rRelease_MatrixAPI()
        self.reset()
        self.api = None
        return

    def connect(self):
        if self.api is None:
            return -1

        return self.api.rInit_MatrixAPI()

    def release(self):
        if self.api is None:
            return -1
        return self.api.rRelease_MatrixAPI()

    def count(self):
        if self.api is None:
            return -1

        return self.api.rDongle_Count(85)

    def mem_size(self, port_no):
        if self.api is None:
            return -1
        return self.api.rDongle_MemSize(port_no, 85)

    def model(self, port_no):
        if self.api is None:
            return -1
        return self.api.rDongle_Model(port_no, 85)

    def version(self, port_no):
        if self.api is None:
            return -1
        return self.api.rDongle_Version(port_no, 85)

    def api_version(self):
        if self.api is None:
            return -1
        return self.api.rGetVersionAPI()

    def drv_version(self):
        if self.api is None:
            return -1
        return self.api.rGetVersionDRV()

    def ser_no(self, user_code, port_no):
        if self.api is None:
            return -1
        return self.api.rDongle_ReadSerNr(user_code, port_no, 85)

    def read_data(self, user_code, count, port_no):
        if self.api is None:
            return -1, []

        fields = [0] * count
        data = (c_int * count)(*fields)

        ret = self.api.rDongle_ReadData(user_code, data, count, port_no, 85)
        if ret == count:
            return ret, list(data)

        return ret, []

    def read_data_ex(self, user_code, pos, count, port_no):
        if self.api is None:
            return -1, []

        fields = [0] * count
        data = (c_int * count)(*fields)

        ret = self.api.rDongle_ReadDataEx(user_code, data, pos, count, port_no, 85)
        if ret == count:
            return ret, list(data)

        return ret, []

    def get_key_flag(self, user_code, port_no):
        return self.api.rDongle_GetKeyFlag(user_code, port_no, 85)

    def encrypt_data(self, user_code, data, port_no):
        if self.api is None:
            return -1, []

        if not isinstance(data, list) or len(data) != 2:
            return -1, []

        plain = (c_int * len(data))(*data)
        ret = self.api.rDongle_EncryptData(user_code, plain, port_no, 85)
        if ret > 0:
            return ret, list(plain)

        return ret, []

    def decrypt_data(self, user_code, data, port_no):
        if self.api is None:
            return -1, []

        if not isinstance(data, list) or len(data) != 2:
            return -1, []

        enc_data = (c_int * len(data))(*data)
        ret = self.api.rDongle_DecryptData(user_code, enc_data, port_no, 85)
        if ret > 0:
            return ret, list(enc_data)

        return ret, []

    def get_driver_flag(self, user_code, port_no):
        if self.api is None:
            return -1
        return self.api.rDongle_GetDriverFlag(user_code, port_no, 85)

    def setconfig_matrixnet(self, access, server_file):
        if self.api is None:
            return -1

        file = bytes(server_file, 'utf-8')
        return self.api.rSetConfig_MatrixNet(access, file)

    def getconfig_matrixnet(self, category):
        if self.api is None:
            return -1

        return self.api.GetConfig_MatrixNet(category)

    def login(self, user_code, app_slot, port_no):
        if self.api is None:
            return -1

        return self.api.rLogIn_MatrixNet(user_code, app_slot, port_no)

    def logout(self, user_code, app_slot, port_no):
        if self.api is None:
            return -1

        return self.api.rLogOut_MatrixNet(user_code, app_slot, port_no)

    def set_ip(self, ip):
        if self.api is None:
            return -1

        if platform.system() == 'Windows':
            return self.api.MxNet_SetIP(ip)
        else:
	        return self.api.MxNet_SetIP(c_char_p(ip.encode('utf-8')))


    def set_port(self, port):
        if self.api is None:
            return -1

        return self.api.MxNet_SetPort(port)

    def reset(self):
        if self.api is None:
            return -1

        return self.api.Reset_MatrixAPI()
