# -*- coding: utf-8 -*-

import mxnet2api
import time
from threading import Thread
import configparser
import os
import platform


class License:

    def __init__(self):

        self.USER_CODE = 1234
        self.APPSLOT = 7
        self.INTERVAL = 5
        self.IP = ""
        self.PORT = 0

        if platform.system() == "Windows":
            ini_path = os.getcwd() + "\\mxnetapi.ini"
        else:
            ini_path = os.getcwd() + "/mxnetapi.ini"
#       print(ini_path)
        config = configparser.ConfigParser()

        try:
            config.read(ini_path)
            self.USER_CODE = int(config["option"]["usercode"])
        except KeyError as e:
            print("key not found in INI file:" + str(e.args))
            exit(-1)

        try:
            self.APP_SLOT = int(config["option"]["appslot"])
        except KeyError:
            self.APPSLOT = 7
            config["option"]["appslot"] = str(self.APPSLOT)

        try:
            self.INTERVAL = int(config["option"]["interval"])
        except KeyError:
            self.INTERVAL = 5
            config["option"]["interval"] = str(self.INTERVAL)

        with open(ini_path, "w") as f:
            config.write(f)

        try:
            self.IP = config["option"]["ip"]
        except KeyError:
            self.IP = "127.0.0.1"

        try:
            self.PORT = int(config["option"]["port"])
        except KeyError:
            self.PORT = 12300

        self.initOk = False
        self.updateStop = False
        self.schedulerThread = None
        self.mxAPI = None
        self.exitAppCallback = None
        self.loginError = False

    def __del__(self):
        try:
            self.mxAPI.release()
        except Exception:
            pass

    def set_port(self, port):
        self.PORT = port

    def set_ip(self, ip):
        self.IP = ip

    def set_usercode(self, uc):
        self.USER_CODE = uc

    def set_appslot(self, appslot):
        self.APPSLOT = appslot

    def set_interval(self, interval):
        self.INTERVAL = interval

    def login_update(self):
        ret = self.mxAPI.login(self.USER_CODE, self.APPSLOT, 1)
        print("Login=" + str(ret))
        if ret < 0:
            self.updateStop = True

    def login_update2(self):
        ret = self.mxAPI.login(self.USER_CODE, self.APPSLOT, 1)
        if ret < 0:
            self.mxAPI.release()
        return ret

    def scheduler(self):
        base_time = time.time()
        while True:
            t = Thread(target=self.login_update)
            t.start()
            t.join()

            if self.updateStop:
                self.exitAppCallback()
                break

            next_time = round(((base_time - time.time()) % self.INTERVAL) or self.INTERVAL)

            for i in range(next_time):
                if self.updateStop:
                    break
                time.sleep(1)
            if self.updateStop:
                break

    def start_update(self, callback):
        self.exitAppCallback = callback
        self.schedulerThread = Thread(target=self.scheduler)
        self.schedulerThread.start()

    def wait_termination(self):
        self.schedulerThread.join()

    def acquire_license(self):
        ret = self.mxAPI.connect()
        if ret < 0:
            print("failed to connect to MxNet2 server:", ret)
            return

        print("connected:", ret)

        ret = self.mxAPI.count()
        print("count", ret)
        if ret <= 0:
            return

        ret = self.mxAPI.ser_no(self.USER_CODE, 1)
        print("serNr", self.mxAPI.ser_no(self.USER_CODE, 1))
        if ret <= 0:
            return

        #
        # Read Memory
        #
        ret, fields = self.mxAPI.read_data(self.USER_CODE, 15, 1)
        print(" ---- read_data ----")
        if ret == 15:
            for i in fields:
                print(i)
        else:
            return

        ret = self.mxAPI.login(self.USER_CODE, self.APPSLOT, 1)
        print("Login=" + str(ret))
        if ret >= 0:
            self.initOk = True

    def get(self):

        self.mxAPI = mxnet2api.API(self.IP, self.PORT)

        t = Thread(target=self.acquire_license)
        t.start()
        t.join()

        return self.initOk

    def quit(self):
        self.updateStop = True
